import React from "react";
import { keyArrowDown, keyArrowLeft, keyArrowRight, keyArrowUp, keyEnd, keyEnter, keyHome, keySpace, } from "@microsoft/fast-web-utilities";
import Foundation from "@microsoft/fast-components-foundation-react";
import { get } from "lodash-es";
import { canUseDOM } from "exenv-es6";
import { DraggableNavigationTreeItem } from "./navigation-tree-item";
import { DragDropItemType } from "./navigation-tree-item.props";
import { dataSetName, dictionaryLink, MessageSystemDataTypeAction, MessageSystemNavigationTypeAction, MessageSystemType, DataType, } from "@microsoft/fast-tooling";
import manageJss from "@microsoft/fast-jss-manager-react";
import styles from "./navigation.style";
import { getDraggableItemClassName, getDragStartMessage, getDragStartState, getDragEndMessage, getDragHoverState, isActiveItem, } from "./navigation.utilities";
export const navigationId = "fast-tooling-react::navigation";
class Navigation extends Foundation {
    constructor(props) {
        super(props);
        this.handledProps = {
            messageSystem: void 0,
        };
        this.scrollActiveItemIntoView = (e) => {
            var _a, _b, _c;
            if (this.props.scrollIntoView &&
                ((_a = e.data.options) === null || _a === void 0 ? void 0 : _a.originatorId) !== navigationId &&
                e.data.activeDictionaryId) {
                (_c = (_b = this.activeItem) === null || _b === void 0 ? void 0 : _b.current) === null || _c === void 0 ? void 0 : _c.scrollIntoView({
                    block: "center",
                    inline: "center",
                    behavior: "smooth",
                });
            }
        };
        /**
         * Handle messages from the message system
         */
        this.handleMessageSystem = (e) => {
            let activeDictionaryId = e.data.activeDictionaryId;
            let activeNavigationConfigId = e.data.activeNavigationConfigId;
            let dataDictionary = e.data.dataDictionary;
            let navigationDictionary = e.data.navigationDictionary;
            let expandedNavigationConfigItems = this.state.expandedNavigationConfigItems;
            let setState = false;
            switch (e.data.type) {
                case MessageSystemType.initialize:
                    activeDictionaryId = e.data.activeDictionaryId
                        ? e.data.activeDictionaryId
                        : e.data.navigationDictionary[1];
                    activeNavigationConfigId = e.data.activeNavigationConfigId
                        ? e.data.activeNavigationConfigId
                        : e.data.navigationDictionary[0][e.data.navigationDictionary[1]][1];
                    setState = true;
                    break;
                case MessageSystemType.data:
                    activeDictionaryId = e.data.activeDictionaryId
                        ? e.data.activeDictionaryId
                        : this.state.activeDictionaryId;
                    activeNavigationConfigId = e.data.activeNavigationConfigId
                        ? e.data.activeNavigationConfigId
                        : this.state.activeNavigationConfigId;
                    setState = true;
                    break;
                case MessageSystemType.navigation:
                    expandedNavigationConfigItems = this.getUpdatedElementsExpanded(e.data.activeDictionaryId, e.data.activeNavigationConfigId);
                    setState = true;
                    break;
            }
            if (setState) {
                this.setState({
                    activeDictionaryId,
                    activeNavigationConfigId,
                    dataDictionary,
                    expandedNavigationConfigItems,
                    navigationDictionary,
                }, this.scrollActiveItemIntoView.bind(this, e));
            }
        };
        /**
         * Handler for the beginning of the drag
         * - This method removes the dragging item from the data
         */
        this.handleDragStart = (index) => {
            return (dictionaryId) => {
                this.setState(getDragStartState(dictionaryId, index, this.state.dataDictionary, this.state.navigationDictionary));
                this.props.messageSystem.postMessage(getDragStartMessage(dictionaryId, navigationId, this.state.navigationDictionary));
            };
        };
        this.handleDragEnd = () => {
            this.props.messageSystem.postMessage(getDragEndMessage(navigationId, this.state.hoveredItem, this.state.linkedData, this.state.linkedDataLocation, this.props.defaultLinkedDataDroppableDataLocation));
            this.setState({
                hoveredItem: null,
            });
        };
        /**
         * Handles hovering over an item
         */
        this.handleDragHover = (type, dictionaryId, navigationConfigId, index, location) => {
            this.setState(getDragHoverState(dictionaryId, navigationConfigId, type, location, index, this.state.hoveredItem, this.state.navigationDictionary, this.props.defaultLinkedDataDroppableDataLocation));
        };
        /**
         * Handle clicks on a navigation item
         */
        this.handleNavigationItemClick = (dictionaryId, navigationConfigId) => {
            let timer;
            let timesClicked = 0;
            return (event) => {
                timesClicked += 1;
                setTimeout(() => {
                    if (timesClicked === 1) {
                        this.handleNavigationItemSingleClick(dictionaryId, navigationConfigId);
                        clearTimeout(timer);
                    }
                    else if (timesClicked === 2) {
                        this.handleNavigationItemDoubleClick(dictionaryId, navigationConfigId);
                        clearTimeout(timer);
                    }
                    timesClicked = 0;
                }, 200);
            };
        };
        /**
         * Update the active item
         */
        this.handleNavigationItemSingleClick = (dictionaryId, navigationConfigId) => {
            this.triggerNavigationUpdate(dictionaryId, navigationConfigId);
        };
        /**
         * Allows editing of the active item
         */
        this.handleNavigationItemDoubleClick = (dictionaryId, navigationConfigId) => {
            this.triggerNavigationEdit(dictionaryId, navigationConfigId);
        };
        /**
         * Update the active items display text
         */
        this.handleNavigationItemChangeDisplayText = (dictionaryId) => {
            return (e) => {
                this.props.messageSystem.postMessage({
                    type: MessageSystemType.data,
                    action: MessageSystemDataTypeAction.update,
                    dictionaryId,
                    dataLocation: dataSetName,
                    data: e.target.value,
                    options: {
                        originatorId: navigationId,
                    },
                });
            };
        };
        /**
         * Update the active items display text focus state
         */
        this.handleNavigationItemBlurDisplayTextInput = () => {
            return () => {
                this.setState({
                    textEditing: null,
                });
            };
        };
        /**
         * Handles key up on the active items display text
         */
        this.handleNavigationItemKeyDownDisplayTextInput = () => {
            return (e) => {
                if (e.target === e.currentTarget) {
                    switch (e.key) {
                        case keyEnter:
                            this.setState({
                                textEditing: null,
                            });
                    }
                }
            };
        };
        /**
         * Update an items expandable state
         */
        this.handleNavigationItemExpandClick = (dictionaryId, navigationConfigId) => {
            return () => {
                this.triggerExpandCollapse(dictionaryId, navigationConfigId);
            };
        };
        /**
         * Handles key up on a tree item
         */
        this.handleNavigationItemKeyDown = (dictionaryId, navigationConfigId) => {
            return (e) => {
                e.preventDefault();
                if (e.target === e.currentTarget) {
                    switch (e.key) {
                        case keyEnter:
                        case keySpace:
                            if (e.target === e.currentTarget) {
                                this.triggerExpandCollapse(dictionaryId, navigationConfigId);
                                this.triggerNavigationUpdate(dictionaryId, navigationConfigId);
                            }
                            break;
                        case keyArrowDown:
                            this.focusNextTreeItem(dictionaryId, navigationConfigId);
                            break;
                        case keyArrowUp:
                            this.focusPreviousTreeItem(dictionaryId, navigationConfigId);
                            break;
                        case keyArrowRight:
                            this.focusAndOpenTreeItems(dictionaryId, navigationConfigId);
                            break;
                        case keyArrowLeft:
                            this.focusAndCloseTreeItems(dictionaryId, navigationConfigId);
                            break;
                        case keyHome:
                            this.focusFirstTreeItem();
                            break;
                        case keyEnd:
                            this.focusLastTreeItem();
                            break;
                    }
                }
            };
        };
        this.messageSystemConfig = {
            onMessage: this.handleMessageSystem,
            config: {
                displayTextDataLocation: dataSetName,
            },
        };
        if (props.messageSystem !== undefined) {
            props.messageSystem.add(this.messageSystemConfig);
        }
        this.state = {
            navigationDictionary: null,
            dataDictionary: null,
            activeDictionaryId: "",
            activeNavigationConfigId: "",
            textEditing: null,
            expandedNavigationConfigItems: {},
            linkedData: void 0,
            linkedDataLocation: null,
            hoveredItem: null,
        };
        this.rootElement = React.createRef();
        this.editableElement = React.createRef();
        this.activeItem = React.createRef();
    }
    render() {
        return (React.createElement("div", { ref: this.rootElement, role: "tree", className: this.props.managedClasses.navigation },
            React.createElement("div", { role: "treeitem", className: this.props.managedClasses.navigation_item, "aria-expanded": this.getExpandedState() }, this.renderDictionaryItem(this.state.navigationDictionary !== null
                ? this.state.navigationDictionary[1]
                : null, 0))));
    }
    componentWillUnmount() {
        if (this.props.messageSystem !== undefined) {
            this.props.messageSystem.remove(this.messageSystemConfig);
        }
    }
    getActiveConfigIds(activeDictionaryId, activeNavigationConfigId) {
        const updatedDictionaryItemConfigItems = new Set(this.state.expandedNavigationConfigItems[activeDictionaryId]);
        if (this.state.expandedNavigationConfigItems[activeDictionaryId] === undefined) {
            return new Set([activeNavigationConfigId]);
        }
        else if (updatedDictionaryItemConfigItems.has(activeNavigationConfigId)) {
            updatedDictionaryItemConfigItems.delete(activeNavigationConfigId);
            return updatedDictionaryItemConfigItems;
        }
        updatedDictionaryItemConfigItems.add(activeNavigationConfigId);
        return updatedDictionaryItemConfigItems;
    }
    renderDictionaryItem(dictionaryId, index) {
        if (dictionaryId !== null) {
            return this.renderNavigationConfig(dictionaryId, this.state.navigationDictionary[0][dictionaryId][1], index);
        }
        return null;
    }
    renderNavigationConfig(dictionaryId, navigationConfigId, index) {
        var _a, _b, _c, _d, _e, _f;
        const schema = (_d = (_c = (_b = (_a = this.state.navigationDictionary[0]) === null || _a === void 0 ? void 0 : _a[dictionaryId]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c[navigationConfigId]) === null || _d === void 0 ? void 0 : _d.schema;
        const isLinkedData = this.state.navigationDictionary[0][dictionaryId] !== undefined &&
            this.state.navigationDictionary[0][dictionaryId][1] === navigationConfigId;
        const isRootLinkedData = this.state.navigationDictionary[1] === dictionaryId &&
            navigationConfigId === "";
        const isDraggable = isLinkedData && !isRootLinkedData; // is linked data and not the root level item
        const isBlockedFromBeingDroppable = !(((_f = (_e = this.props) === null || _e === void 0 ? void 0 : _e.droppableBlocklist) === null || _f === void 0 ? void 0 : _f.includes(schema === null || schema === void 0 ? void 0 : schema.$id)) || false); // is included in the droppable blocked list provided
        const isDroppable = isBlockedFromBeingDroppable &&
            ((isLinkedData && (schema === null || schema === void 0 ? void 0 : schema.type) === DataType.object && !isRootLinkedData) || // a piece of linked data that is not the root and is an object type
                (schema === null || schema === void 0 ? void 0 : schema[dictionaryLink]) || // an identified dictionary link
                (isRootLinkedData && this.props.defaultLinkedDataDroppableDataLocation)); // the root linked data with an defined droppable data location
        const isTriggerRenderable = this.shouldTriggerRender(dictionaryId, navigationConfigId);
        const content = this.renderContent(dictionaryId, navigationConfigId, isTriggerRenderable);
        const itemType = isRootLinkedData && isDroppable
            ? DragDropItemType.rootLinkedData
            : isRootLinkedData
                ? DragDropItemType.rootLinkedDataUndroppable
                : isLinkedData && isDroppable
                    ? DragDropItemType.linkedData
                    : isLinkedData
                        ? DragDropItemType.linkedDataUndroppable
                        : isDroppable
                            ? DragDropItemType.linkedDataContainer
                            : DragDropItemType.undraggableUndroppable;
        const trigger = isTriggerRenderable
            ? this.renderTrigger(itemType, this.state.navigationDictionary[0][dictionaryId][0][navigationConfigId]
                .text, content !== null, isDraggable, itemType !== DragDropItemType.undraggableUndroppable, dictionaryId, navigationConfigId, index)
            : null;
        if (trigger === null && content === null) {
            return null;
        }
        return (React.createElement("div", { role: "treeitem", className: this.props.managedClasses.navigation_itemRegion, "aria-expanded": this.getExpandedState(dictionaryId, navigationConfigId), "data-type": itemType, key: index },
            trigger,
            content));
    }
    renderTrigger(type, text, isCollapsible, isDraggable, isDroppable, dictionaryId, navigationConfigId, index) {
        return (React.createElement(DraggableNavigationTreeItem, { type: type, index: index, key: index, isCollapsible: isCollapsible, isEditing: this.isEditing(dictionaryId, navigationConfigId), inputRef: this.editableElement, itemRef: isActiveItem(this.state.activeDictionaryId, dictionaryId, this.state.activeNavigationConfigId, navigationConfigId)
                ? this.activeItem
                : null, className: getDraggableItemClassName(isCollapsible, isDraggable, isDroppable, dictionaryId, navigationConfigId, this.state.hoveredItem, this.state.activeDictionaryId, this.state.activeNavigationConfigId, this.props.defaultLinkedDataDroppableDataLocation, this.props.managedClasses.navigation_item, this.props.managedClasses.navigation_item__expandable, this.props.managedClasses.navigation_item__active, this.props.managedClasses.navigation_item__draggable, this.props.managedClasses.navigation_item__droppable, this.props.managedClasses.navigation_item__hover, this.props.managedClasses.navigation_item__hoverAfter, this.props.managedClasses.navigation_item__hoverBefore), expandTriggerClassName: this.props.managedClasses.navigation_itemExpandTrigger, contentClassName: this.props.managedClasses.navigation_itemContent, displayTextInputClassName: this.props.managedClasses.navigation_itemDisplayTextInput, handleExpandClick: this.handleNavigationItemExpandClick(dictionaryId, navigationConfigId), handleClick: this.handleNavigationItemClick(dictionaryId, navigationConfigId), handleInputChange: this.handleNavigationItemChangeDisplayText(dictionaryId), handleInputBlur: this.handleNavigationItemBlurDisplayTextInput(), handleInputKeyDown: this.handleNavigationItemKeyDownDisplayTextInput(), handleKeyDown: this.handleNavigationItemKeyDown(dictionaryId, navigationConfigId), dictionaryId: dictionaryId, navigationConfigId: navigationConfigId, dragStart: this.handleDragStart(index), dragEnd: this.handleDragEnd, dragHover: this.handleDragHover, text: text }));
    }
    renderContent(dictionaryId, navigationConfigId, isTriggerRendered) {
        const navigationConfig = this.state.navigationDictionary[0][dictionaryId][0][navigationConfigId];
        if (Array.isArray(navigationConfig.items) && navigationConfig.items.length > 0) {
            const content = navigationConfig.items.map((navigationConfigItemId, index) => {
                if (navigationConfig.schema[dictionaryLink] &&
                    Array.isArray(navigationConfig.data) &&
                    navigationConfig.data[index]) {
                    return this.renderDictionaryItem(navigationConfig.data[index].id, index);
                }
                return this.renderNavigationConfig(dictionaryId, navigationConfigItemId, index);
            });
            const isEmpty = content.find((contentItem) => {
                return contentItem !== null;
            }) === undefined;
            if (!isEmpty) {
                if (isTriggerRendered) {
                    return (React.createElement("div", { role: "group", key: "content", className: this.props.managedClasses.navigation_itemList }, content));
                }
                return content;
            }
        }
        return null;
    }
    /**
     * Determine if an element is currently being edited
     */
    isEditing(dictionaryId, navigationConfigId) {
        return (this.state.textEditing &&
            this.state.textEditing.dictionaryId === dictionaryId &&
            this.state.textEditing.navigationConfigId === navigationConfigId &&
            this.state.textEditing.navigationConfigId === "");
    }
    shouldTriggerRender(dictionaryId, navigationConfigId) {
        return (!Array.isArray(this.props.types) ||
            this.props.types.includes(this.state.navigationDictionary[0][dictionaryId][0][navigationConfigId]
                .type));
    }
    getExpandedState(dictionaryId, navigationConfigId) {
        // assume this is the root level tree item if these are undefined
        if (dictionaryId === undefined && navigationConfigId === undefined) {
            if (this.state.navigationDictionary === null) {
                return false;
            }
            return (this.state.expandedNavigationConfigItems[this.state.navigationDictionary[1]] !== undefined &&
                this.state.expandedNavigationConfigItems[this.state.navigationDictionary[1]].has(this.state.navigationDictionary[0][this.state.navigationDictionary[1]][1]));
        }
        return (this.state.expandedNavigationConfigItems[dictionaryId] !== undefined &&
            this.state.expandedNavigationConfigItems[dictionaryId].has(navigationConfigId));
    }
    getUpdatedElementsExpanded(dictionaryId, navigationConfigId) {
        return Object.assign(Object.assign({}, this.state.expandedNavigationConfigItems), this.getParentElement(dictionaryId));
    }
    getParentElement(dictionaryId) {
        if (this.state.dataDictionary[0][dictionaryId].parent) {
            const parentDictionaryId = this.state.dataDictionary[0][dictionaryId].parent
                .id;
            const parentDictionaryItem = this.state.expandedNavigationConfigItems[parentDictionaryId]
                ? new Set([
                    "",
                    ...this.state.expandedNavigationConfigItems[parentDictionaryId],
                ])
                : new Set([""]);
            const parentDictionaryItemDataLocations = new Set(this.state.dataDictionary[0][dictionaryId].parent.dataLocation.split("."));
            return Object.assign({ [parentDictionaryId]: new Set([
                    ...parentDictionaryItemDataLocations,
                    ...parentDictionaryItem,
                ]) }, this.getParentElement(parentDictionaryId));
        }
        return {};
    }
    triggerExpandCollapse(dictionaryId, navigationConfigId) {
        this.setState({
            expandedNavigationConfigItems: Object.assign(Object.assign({}, this.state.expandedNavigationConfigItems), { [dictionaryId]: this.getActiveConfigIds(dictionaryId, navigationConfigId) }),
        });
    }
    triggerNavigationEdit(dictionaryId, navigationConfigId) {
        this.setState({
            textEditing: {
                dictionaryId,
                navigationConfigId,
            },
        }, () => {
            var _a;
            if ((_a = this.editableElement) === null || _a === void 0 ? void 0 : _a.current) {
                this.editableElement.current.focus();
                this.editableElement.current.select();
            }
        });
    }
    triggerNavigationUpdate(dictionaryId, navigationConfigId) {
        this.setState({
            textEditing: null,
        }, () => {
            this.props.messageSystem.postMessage({
                type: MessageSystemType.navigation,
                action: MessageSystemNavigationTypeAction.update,
                activeDictionaryId: dictionaryId,
                activeNavigationConfigId: navigationConfigId,
                options: {
                    originatorId: navigationId,
                },
            });
        });
    }
    findCurrentTreeItemIndex(nodes, dictionaryId, navigationConfigId) {
        return nodes.findIndex((node) => {
            return (node.dataset.dictionaryid === dictionaryId &&
                node.dataset.navigationconfigid === navigationConfigId);
        });
    }
    focusNextTreeItem(dictionaryId, navigationConfigId) {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            const currentIndex = this.findCurrentTreeItemIndex(nodes, dictionaryId, navigationConfigId);
            const nextIndex = currentIndex !== -1 && currentIndex !== nodes.length - 1
                ? currentIndex + 1
                : nodes.length - 1;
            nodes[nextIndex].focus();
        }
    }
    focusPreviousTreeItem(dictionaryId, navigationConfigId) {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            const currentIndex = this.findCurrentTreeItemIndex(nodes, dictionaryId, navigationConfigId);
            const previousIndex = currentIndex !== -1 && currentIndex !== 0 ? currentIndex - 1 : 0;
            nodes[previousIndex].focus();
        }
    }
    focusFirstTreeItem() {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            nodes[0].focus();
        }
    }
    focusLastTreeItem() {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            nodes[nodes.length - 1].focus();
        }
    }
    focusAndOpenTreeItems(dictionaryId, navigationConfigId) {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            const currentIndex = this.findCurrentTreeItemIndex(nodes, dictionaryId, navigationConfigId);
            const ariaExpanded = get(nodes[currentIndex], 'parentElement.parentElement.attributes["aria-expanded"].value');
            if (nodes[currentIndex].tagName !== "A" &&
                ariaExpanded === "true" &&
                nodes[currentIndex + 1]) {
                nodes[currentIndex + 1].focus();
            }
            else if (ariaExpanded === "false") {
                this.triggerExpandCollapse(dictionaryId, navigationConfigId);
                this.triggerNavigationUpdate(dictionaryId, navigationConfigId);
            }
        }
    }
    focusAndCloseTreeItems(dictionaryId, navigationConfigId) {
        if (canUseDOM()) {
            const nodes = this.getTreeItemNodes();
            const currentIndex = this.findCurrentTreeItemIndex(nodes, dictionaryId, navigationConfigId);
            const ariaExpanded = get(nodes[currentIndex], 'parentElement.parentElement.attributes["aria-expanded"].value');
            if (nodes[currentIndex].tagName === "A") {
                const parent = get(nodes[currentIndex], "parentElement.parentElement.parentElement.parentElement.firstChild");
                if (parent) {
                    parent.querySelector("[data-dictionaryid]").focus();
                }
            }
            else if (ariaExpanded === "false" && nodes[currentIndex - 1]) {
                nodes[currentIndex - 1].focus();
            }
            else if (ariaExpanded === "true") {
                this.triggerExpandCollapse(dictionaryId, navigationConfigId);
                this.triggerNavigationUpdate(dictionaryId, navigationConfigId);
            }
        }
    }
    getTreeItemNodes() {
        const nodes = Array.from(this.rootElement.current.querySelectorAll("div[role='treeitem'] > a, div[role='treeitem'] > span > [data-dictionaryid]"));
        return nodes.filter((node) => node.offsetParent !== null);
    }
}
Navigation.displayName = "Navigation";
Navigation.defaultProps = {
    managedClasses: {},
    messageSystem: void 0,
    scrollIntoView: false,
};
export { Navigation };
export default manageJss(styles)(Navigation);
